static char sccsid[]="%Z% %M% %I% %E% %U%";
/************************************************************************/
/*																		*/
/*	akxssrch.c															*/
/*																		*/
/*	쐬 : 2001/02/15 Akito Kobayashi									*/
/*	XV : 2019/08/23 Akito Kobayashi Add akxs_seqr_str(),				*/
/*								,akxs_seqr_str_num(),akxs_seqr_str2()	*/
/*	XV : 2019/09/09 Akito Kobayashi Add akxs_iseq(),akxs_ins_sseq()   */
/*        ,akxs_ins_sseq_array(),akxs_ins_shseq(),akxs_ins_shseq_array()*/
/************************************************************************/
/*	  error code : -185210101`-185219999		*/
#include "akxcommon.h"

static int _seqsr();
static int _seqss();
static int _seqsd();
static int _seqsi();

/*	01	*/
int akxs_seqs(func,hashbp)
char func;
HASHB *hashbp;
{
	int i;

	if (!hashbp) return -185210101;
/*
printf("akxs_seqs: func=%c hashbp->ha_prereg=%08x\n",func,hashbp->ha_prereg);
*/
	switch (func) {
	  case 'r':
	  case 'R':
			i=_seqsr(hashbp);
			break;
	  case 's':
	  case 'S':
			i=_seqss(hashbp);
			break;
	  case 'd':
	  case 'D':
			i=_seqsd(hashbp);
			break;
	  case 'i':
	  case 'I':
			i=_seqsi(hashbp);
			break;
	  default:
			i=-2;
	}
	return i;
}

/*	02	*/
static int _seqsi(hp)
HASHB *hp;
{
	int mx,i,j,klen,len1,aux;
	char *rp,**rpp;

	i = 0;	/* 2021.7.17 add mx */
	if ((mx=hp->ha_maxreg)<=0) i -= 4;
	if (!hp->ha_reg) i -= 8;
	if (!hp->ha_next) i -= 16;

	if (i >= 0) {
		klen = hp->ha_keylen;
		rp = hp->ha_reg;
		if (klen > 0) {
			len1 = klen + 1;
			for (j=0;j<mx;j++) {
				*rp = 0;
				rp += len1;
			}
		}
		else {
			memset(rp,0,(mx+1)*(sizeof(int)+1));
			if (!((aux=hp->ha_aux) & (AKX_HASX_OPT_USE_MALLOC | AKX_HASX_OPT_TMP_MALLOC))) {
				rpp = (char **)rp;
				if (aux & AKX_HASX_OPT_RESET_MEM) akxm_cct_reset((tdtCONSTCT *)rpp[0]);
				else rpp[0] = (char *)akxm_cct_mem_new(mx*32);
			}
		}
		memset(hp->ha_next,0,mx+1);
		hp->ha_aux = 0;
	}
	if (i < 0) i += -18521020;
	return i;
}

/*	03	*/
static int _seqss(hp)
HASHB *hp;
{
	int i,lklen,opt,lalen;
	char *p,*kp,**rpp,*np,*rp;
	short klen;
	tdtAllocCtl *pAC;
	char  *(*m_alloc)();
	tdtCONSTCT *pConstCt;

	if (i = _seqsr(hp)) return i;

	np = (char *)hp->ha_next;
/*
printf("_seqss: hp->ha_aux=%d\n",hp->ha_aux);
*/
#if 1	/* 2021.7.17 */
	for (i=1;i<=hp->ha_aux;i++) {
		if (!np[i]) break;
	}
	if (i > hp->ha_maxreg) return 0;
	if (i > hp->ha_aux) hp->ha_aux++;
/*
printf("_seqss: i=%d hp->ha_aux=%d\n",i,hp->ha_aux);
*/
#else
	for (i=0;i<hp->ha_aux;i++) {
		if (!np[i]) break;
	}
	if (i >= hp->ha_maxreg) return 0;
	if (i >= hp->ha_aux) hp->ha_aux++;
#endif
	rp = hp->ha_reg;
	rpp = (char **)rp;
	klen = hp->ha_keylen;
	kp = hp->ha_key;
	if (klen > 0) {
		p = rp + (i-1)*(klen+1);
		memzcpy(p,kp,klen);
/*
printf("_seqss: p=[%s]\n",p);
*/
		np[i] = 1;
	}
	else {
		if (!klen) {
			lalen = strlen(kp) + 1;
		}
		else if (klen > 0) {
			lalen = klen;
		}
		else {
			memcpy(&lklen,kp,sizeof(int));
			if (lklen < 0) return -32;
			lalen = lklen + sizeof(int);
		}
		if (hp->ha_id[1] == 'M') {
			pAC = (tdtAllocCtl *)hp->ha_reg;
			m_alloc = pAC->ac_malloc;
			pConstCt = pAC->ac_constct;
			p = akxm_malloc_constct(m_alloc,pConstCt,lalen);
		}
		else if (rpp[0]) {
			p = akxm_cct_malloc(rpp[0],lalen);
		}
		else {
			p = rpp[i];
			if (p) p = Realloc(p,lalen);
			else p = Malloc(lalen);
		}
		if (p) {
			if (klen < 0) {
				memcpy(p,&lklen,sizeof(int));
				memcpy(p+sizeof(int),kp,lklen);
			}
			else memcpy(p,kp,lalen);
		/*	if (opt = hp->ha_prereg) opt >>= 16;	*/
			np[i] = 1/* | opt*/;
			rpp[i] = p;
/*
printf("seqss:i=%d kp=[%s] np[i]=%08x\n",i,p,np[i]);
*/
		}
		else i = -33;
	}
	return i;		/* 2021.7.17 i+1 --> i */
}

/*	04	*/
static int _seqsr(hp)
HASHB *hp;
{
	int i;
	char *p,*kp,**rpp,*np,*rp;
	short len,klen;
	int  ldlen,lklen,opt;

	rp = hp->ha_reg;
	rpp = (char **)rp;
	np = (char *)hp->ha_next;
	if (!(kp = hp->ha_key)) return -185210401;
	if ((klen=hp->ha_keylen) < 0) {
		memcpy(&lklen,kp,sizeof(int));
		if (lklen < 0) return -185210402;
		kp += sizeof(int);
	}
/*
printf("seqr: klen=%d kp=[%s]\n",klen,kp);
*/
	opt = hp->ha_prereg;
#if 1	/* 2021.7.17 */
	for (i=1;i<=hp->ha_aux;i++) {
#else
	for (i=0;i<hp->ha_aux;i++) {
#endif
		if (!np[i]) ;
		else if (klen > 0) {
			p = rp + (i-1)*(klen+1);
			if (!akxs_opt_memcmp(p,kp,klen,opt)) break;
		}
		else {
			if (!(p = rpp[i])) ;
			else if (!klen) {
/*
printf("seqr:i=%d p=[%s] kp=[%s] opt=%08x\n",i,p,kp,opt);
*/
				if (!akxs_opt_strcmp(p,kp,opt)) break;
			}
			else {
				memcpy(&ldlen,p,sizeof(int));
				if (lklen == ldlen) {
					if (!akxs_opt_memcmp(p+sizeof(int),kp,lklen,opt)) break;
				}
			}
		}
	}
#if 1	/* 2021.7.17 */
	if (i > hp->ha_aux) i = 0;
#else
	if (i >= hp->ha_aux) i = -1;
#endif

	return i;		/* 2021.7.17 i+1 --> i */
}

/*	05	*/
static int _seqsd(hp)
HASHB *hp;
{
	int i;
	char *np;

	i = hp->ha_hix;
	if (i > 0 ) {
		if (i > hp->ha_aux) return -185210501;
	}
	else {
		if ((i=_seqsr(hp)) <= 0) return i;
	}
	np = (char *)hp->ha_next;
	np[i] = 0;		/* 2021.7.17 i-1 --> i */
	return i;
}

/*	06	*/
/********1*********2*********3*********4*********5*********7*/
/*  : iOpt    : IvV								*/
/*					0x01 : ignore case						*/
/*					0x02 : ignore p/{p					*/
/*					0x08 : not use akxm_cct_malloc()		*/
/*						   m_alloc<>NULL ̂Ƃ́Aoff	*/
/* ԋp : nbV\									*/
/*		  ha_id[0] = 'S' ignore case  ̂Ƃ́A's'			*/
/*		  ha_id[1] = 'Q' m_alloc<>NULL̂Ƃ́A'M'			*/
/************************************************************/
HASHB *akxs_seqsm_new(sKeyLen,lMaxReg,iOpt,m_alloc,pConstCt)
short sKeyLen;
int  lMaxReg,iOpt;
char    *(*m_alloc)();
tdtCONSTCT *pConstCt;
{
	HASHB *tph;
	int l;
	char c,**rpp;
	tdtAllocCtl *pAC;

	if (lMaxReg <= 0) return NULL;
	if (!(tph=(HASHB *)akxm_malloc_constct(m_alloc,pConstCt,sizeof(HASHB)))) return NULL;
	memset(tph,0,sizeof(HASHB));
	c = 'S';
	if (iOpt & AKX_HASX_OPT_CASE_KEY) c = tolower(c);
	tph->ha_id[0] = c;
	c = 'Q';
	if (m_alloc) {
		c = 'M';
		iOpt &= AKX_HASX_OPT_USE_MALLOC;
		iOpt |= AKX_HASX_OPT_TMP_MALLOC;
	}
	tph->ha_id[1] = c;
	tph->ha_keylen = sKeyLen;
	tph->ha_maxreg = lMaxReg;
	tph->ha_prereg = iOpt;
	tph->ha_aux    = iOpt;
/*
printf("akxs_seqs_new: sKeyLen=%d lMaxReg=%d id[1]=%c\n",sKeyLen,lMaxReg,tph->ha_id[1]);
*/
#if 1	/* 2021.9.16 */
	if (sKeyLen > 0) l = lMaxReg*(sKeyLen+1);
	else l = (lMaxReg+1)*(sizeof(char *)+1);
	if (!(tph->ha_reg=akxm_malloc_constct(m_alloc,pConstCt,l))) {
#else
	if (!(tph->ha_reg=Malloc((lMaxReg+1)*sizeof(char *)))) {	/* 2021.7.17 lMaxReg --> lMaxReg+1 */
#endif
		akxs_seqs_free(tph);
		return NULL;
	}
#if 1	/* 2021.9.16 */
	if (!(tph->ha_next=(int *)akxm_malloc_constct(m_alloc,pConstCt,lMaxReg+1))) {
#else
	if (!(tph->ha_next=(int *)Malloc(lMaxReg+1))) {	/* 2021.7.17 lMaxReg --> lMaxReg+1 */
#endif
		akxs_seqs_free(tph);
		return NULL;
	}
	if (akxs_seqs('i',tph)) {
		akxs_seqs_free(tph);
		return NULL;
	}
	/* (iOpt & AKX_HASX_OPT_USE_MALLOC)==0 && (iOpt & AKX_HASX_OPT_TMP_MALLOC)==0 ̂Ƃ́A
	    LŁArpp[0]́Aakxm_cct_mem_new()ŁAݒ肳Ă */
	if (m_alloc) {
		if (!(pAC = (tdtAllocCtl *)akxm_malloc_constct(m_alloc,pConstCt,sizeof(tdtAllocCtl)))) {
			errno = -1107;
			akxs_seqs_free(tph);
			return NULL;
		}
		pAC->ac_malloc = m_alloc;
		pAC->ac_constct = pConstCt;
		rpp = (char **)tph->ha_reg;
		rpp[0] = (char *)pAC;
	}
	/* rpp[0] ==NULL : keyMalloc()Bklenkey̕ۑ`ςB
	   m_alloc==NULL : rpp[0] : akxm_cct_mem_new()ŁAݒ肳ĂB
	   m_alloc<>NULL : rpp[0] : pACɂȂĂB */
	return tph;
}

/*	07	*/
HASHB *akxs_seqs_new(sKeyLen,lMaxReg)
short sKeyLen;
int  lMaxReg;
{
	return akxs_seqsm_new(sKeyLen,lMaxReg,0,NULL,NULL);
}

/*	08	*/
int akxs_seqs_free(hp)
HASHB *hp;
{
#if 1	/* 2021.7.17 */
	int i;
	char **rp;
#endif
	if (hp) {
#if 1	/* 2021.9.16 */
		if (hp->ha_id[1]=='M' && hp->ha_reg) {
			akxs_free_reg(hp->ha_maxreg,hp->ha_keylen,hp->ha_reg);
		}
#else
#if 1	/* 2021.7.17 */
		if (rp = (char **)hp->ha_reg) {
			for (i=1;i<=hp->ha_aux;i++) {
				if (rp[i]) Free(rp[i]);
			}
			Free(rp);
		}
#else
		if (hp->ha_reg) Free(hp->ha_reg);
		if (hp->ha_next) Free(hp->ha_next);
#endif
#endif
		Free(hp);
	}
	return 0;
}

static int _xseqs_chk();
static int _xseqs_used();
static int _xseqs_max();

/*	11	*/
/********1*********2*********3*********4*********5*********7*/
/*  : iOpt    : IvV								*/
/*					0x01 : ignore case						*/
/*					0x02 : ignore p/{p					*/
/*					0x08 : not use akxm_cct_malloc()		*/
/*						   m_alloc<>NULL ̂Ƃ́Aoff	*/
/************************************************************/
XHASHB *akxs_xseqsm_new2(sKeyLen,lMaxReg,lDatLen,iOpt,m_alloc,pConstCt)
short sKeyLen;
int  lMaxReg,lDatLen,iOpt;
char    *(*m_alloc)();
tdtCONSTCT *pConstCt;
{
	XHASHB *p;
	int l;
	char c;

	if (lMaxReg <= 0) {
		errno = -185211101;
		return NULL;
	}
	if (!(p=(XHASHB *)akxm_malloc_constct(m_alloc,pConstCt,sizeof(XHASHB)))) {
		errno = -185211102;
		return NULL;
	}

	if (sKeyLen>0 && sKeyLen<=sizeof(long)) c = 'L';
	else if (sKeyLen>sizeof(long) && sKeyLen<=sizeof(long)*2) c = '2';
	else c = 'X';
/*
printf("akxs_xseqs_new2: sKeyLen=%d lMaxReg=%d lDatLen=%d id[1]=%c\n",sKeyLen,lMaxReg,lDatLen,c);
*/
	if (m_alloc) {
		iOpt &= ~AKX_HASX_OPT_USE_MALLOC;
		iOpt |= AKX_HASX_OPT_TMP_MALLOC;
	}
	p->xha_id[0] = 'S';
	p->xha_id[1] = c;
	p->xha_keylen = sKeyLen;
	p->xha_maxreg = lMaxReg;
	p->xha_prereg = 0;
	p->xha_datlen = lDatLen;
	p->xha_option = iOpt;
	p->xha_xhix   = 0;
	p->xha_hashb  = NULL;
	p->xha_xhnext = NULL;
	p->xha_datreg = NULL;
	p->xha_malloc = m_alloc;
	p->xha_constct= pConstCt;
	return p;
}

/*	12	*/
XHASHB *akxs_xseqs_new2(sKeyLen,lMaxReg,lDatLen,iOpt)
short sKeyLen;
int  lMaxReg,lDatLen,iOpt;
{
	return akxs_xseqsm_new2(sKeyLen,lMaxReg,lDatLen,iOpt,NULL,NULL);
}

/*	13	*/
XHASHB *akxs_xseqs_new(sKeyLen,lMaxReg)
short sKeyLen;
int  lMaxReg;
{
	return akxs_xseqs_new2(sKeyLen,lMaxReg,-2,0);
}

/*	14	*/
int akxs_xseqs_free(tp_xseqsb)
XHASHB *tp_xseqsb;
{
	HASHB *tph;
	char *p;

	if (tp_xseqsb) {
		if (tph=tp_xseqsb->xha_hashb) {
			akxs_seqs_free(tph);
		}
		if (!tp_xseqsb->xha_malloc && (p=tp_xseqsb->xha_datreg)) {
			akxs_free_reg(tp_xseqsb->xha_maxreg,tp_xseqsb->xha_datlen,p);
		}
		akxs_xseqs_free(tp_xseqsb->xha_xhnext);
		Free(tp_xseqsb);
	}
	return 0;
}

/*	15	*/
int akxs_xseqs2(tp_xseqsb,cCmnd,cpKey,cppDat)
XHASHB *tp_xseqsb;
char cCmnd, *cpKey;
char **cppDat;		/* cCmnd='S'or's'̂Ƃ́A(char *)Ɠǂݑւ */
{
	XHASHB *tpcur,*tpnext;
	int l,i,offset,len;
	HASHB *tph,ha;
	int ret,opt;
	char cCmd,c,cc;
/*
printf("akxs_xseqs2:Enter cCmnd=%c cpKey=[%s] cppDat=[%s]\n",cCmnd,cpKey,cppDat);
*/
	if (!tp_xseqsb) return -185211501;
	cCmd = akxcupper(cCmnd);
	if ((cCmd=='R'||cCmd=='P'||cCmd=='K') && cppDat) *cppDat = NULL;
	switch (cCmd) {
		case 'R':
		case 'S':
		case 'D':
			if (tp_xseqsb->xha_xhix) return _xseqs_chk(tp_xseqsb,cCmd,cpKey,cppDat);
			break;
		case 'K':
		case 'P':
			return _xseqs_chk(tp_xseqsb,cCmd,cpKey,cppDat);
		case 'U':
			return _xseqs_used(tp_xseqsb);
		case 'M':
			return _xseqs_max(tp_xseqsb);
		default:
			return -185211502;
	}
	if (!cpKey) return -185211503;
	l = 0;
	offset = 0;
	tpnext = tp_xseqsb;
	while (tpnext) {
		tpcur = tpnext;
		if (!(tph=tpcur->xha_hashb)) {
			if (cCmd == 'R' || cCmd == 'D') return 0;
			if (tph=akxs_seqsm_new(tpcur->xha_keylen,tpcur->xha_maxreg,
			                       tpcur->xha_option,tpcur->xha_malloc,tpcur->xha_constct))
				tpcur->xha_hashb = tph;
			else return errno;
			tph->ha_prereg = tpcur->xha_prereg;
			cc = tpcur->xha_id[0];
			c = toupper(cc);
			if (c != cc) tph->ha_prereg |= AKX_ARGV_USE_ICMP;
/*
printf("akxs_xseqs2: cCmd=%c tph->ha_prereg=%08x\n",cCmd,tph->ha_prereg);
*/
		}
		tph->ha_key = cpKey;
		tph->ha_hix = 0;
		if (cCmd=='S' && cppDat && tpcur->xha_datlen == -2 && tph->ha_id[1]=='L')
			tph->ha_next = (int *)cppDat;
		if (i=akxs_seqs(cCmd,tph)) {
/*
printf("akxs_xseqs2:akxs_seqs i=%d\n",i);
*/
			if (i>0) {
				if ((ret=akxs_dreg_proc(tpcur,cCmd,i,cppDat))<0) return ret;
				i += offset;
/*
printf("akxs_xseqs2:akxs_dreg_proc i=%d\n",i);
*/
			}
			return i;
		}
		tpnext = tpcur->xha_xhnext;
		offset += tpcur->xha_maxreg;
	}
	if (cCmd == 'R' || cCmd == 'D') return i;
	if (!(tpnext=(XHASHB *)akxm_malloc_constct(tpcur->xha_malloc,tpcur->xha_constct,sizeof(XHASHB)))) return -76;
	tpcur->xha_xhnext = tpnext;
	memcpy(tpnext,tpcur,sizeof(XHASHB));
	tpnext->xha_xhix = l;
	tpnext->xha_hashb = NULL;
	tpnext->xha_xhnext = NULL;
	tpnext->xha_datreg = NULL;
	if ((i = akxs_xseqs2(tpnext,cCmd,cpKey,cppDat))>0) i += offset;
	tpnext->xha_xhix = 0;
	return i;
}

/*	16	*/
int akxs_xseqs(tp_xseqsb,cCmd,cpKey)
XHASHB *tp_xseqsb;
char cCmd, *cpKey;
{
	return akxs_xseqs2(tp_xseqsb,cCmd,cpKey,NULL);
}

/*	17	*/
static int _xseqs_chk(tp_xseqsb,cCmd,cpKey,cppDat)
XHASHB *tp_xseqsb;
char cCmd, *cpKey,**cppDat;
{
	XHASHB *tpcur;
	int i,offset,max,next,ix;
	HASHB *tph;
	int ret;
	char cCmdw;

	ret = -185211701;
	i = tp_xseqsb->xha_xhix;
	tp_xseqsb->xha_xhix = 0;
	offset = 0;
	tpcur = tp_xseqsb;
	max = tpcur->xha_maxreg;
	while (tpcur) {
		next = offset + max;
		if (next >= i) {
			ret = 0;
			if (tph=tpcur->xha_hashb) {
				tph->ha_hix = ix = i - offset;
				if (cCmd != 'P') tph->ha_key = cpKey;
				else if (cCmd=='S' && cppDat && tpcur->xha_datlen == -2 && tph->ha_id[1]=='L')
					tph->ha_next = (int *)cppDat;
				ret = akxs_seqs(cCmd,tph);
				if (ret > 0) {
					if ((cCmdw=cCmd)=='P' || cCmdw=='K') cCmdw = 'R';
					if ((ret=akxs_dreg_proc(tpcur,cCmdw,ix,cppDat))<0) return ret;
					if ((cCmd == 'P') && cpKey) *(char **)cpKey = tph->ha_key;
					return i;
				}
			}
			break;
		}
		tpcur = tpcur->xha_xhnext;
		offset = next;
	}
	return ret;
}

/*	18	*/
static int _xseqs_used(tp_xseqsb)
XHASHB *tp_xseqsb;
{
	XHASHB *tpcur;
	int ret,count;
	HASHB *tph;

	count = 0;
	tpcur = tp_xseqsb;
	while (tpcur) {
		tpcur->xha_xhix = 0;
		if (tph=tpcur->xha_hashb) {
			ret = akxs_seqs('U',tph);
			tpcur->xha_xhix = ret;
			if (ret > 0) count += ret;
		}
		tpcur = tpcur->xha_xhnext;
	}
	return count;
}

/*	19	*/
static int _xseqs_max(tp_xseqsb)
XHASHB *tp_xseqsb;
{
	XHASHB *tpcur;
	int ret,m;
	HASHB *tph;

	m = 0;
	tpcur = tp_xseqsb;
	while (tpcur) {
		m += tpcur->xha_maxreg;
		tpcur = tpcur->xha_xhnext;
	}
	return m;
}

/*	20	*/
int akxs_seqr_ptr(ptra,max_ptra,ptr)
char *ptra[],*ptr;
int max_ptra;
{
	int i,ii;
/*
printf("akxs_seqr_ptr: ptr=%08x\n",ptr);
*/
	if (!ptra) return -185212001;
	else if (!ptr) return 0;
	ii = 0;
	for (i=0;i<max_ptra;i++) {
		if (ptra[i] == ptr) {
/*
printf("akxs_seqr_ptr: found ptr=%08x i=%d\n",ptr,i);
*/
			ii= i + 1;
			break;
		}
	}
	return ii;
}

/*
typedef struct tdtBINS_LEAF {
	char *keyp;
	char *datp;
	struct tdtBINS_LEAF *left;
	struct tdtBINS_LEAF *right;
} tdtBINS_LEAF;
*/
/*	31	*/
HASHB *akxs_bins_new(sKeyLen,sDatLen,iOpt)
short sKeyLen;
short sDatLen;
int   iOpt;
{
	HASHB *tph;

	if (!(tph=(HASHB *)Malloc(sizeof(HASHB)))) return NULL;
	memset(tph,0,sizeof(HASHB));
	tph->ha_id[0] = 'B';
	tph->ha_id[1] = 'S';
	tph->ha_keylen    = sKeyLen;
	tph->ha_maxreg = sDatLen;
	return tph;
}

/*	32	*/
static void _leaf_free(leaf)
tdtBINS_LEAF *leaf;
{
	if (leaf) {
		if (leaf->keyp) Free(leaf->keyp);
		if (leaf->datp) Free(leaf->datp);
		if (leaf->left)  _leaf_free(leaf->left);
		if (leaf->right) _leaf_free(leaf->right);
/*
printf("_leaf_free: leaf=%08x\n",leaf);
*/
		Free(leaf);
	}
}

/*	33	*/
int akxs_bins_free(tph)
HASHB *tph;
{
	if (tph) {
		if (tph->ha_reg) _leaf_free((tdtBINS_LEAF *)tph->ha_reg);
		Free(tph);
	}
	return 0;
}

/*	34	*/
static int _bins_count_leaf(leaf)
tdtBINS_LEAF *leaf;
{
	tdtBINS_LEAF *next;

	if (leaf) {
		if (next=leaf->left)  {
			_bins_count_leaf(next);
			leaf->count_l = next->count_a + 1;
		}
		if (next=leaf->right) {
			_bins_count_leaf(next);
			leaf->count_r = next->count_a + 1;
		}
		leaf->count_a = leaf->count_l + leaf->count_r;
/*
printf("_bins_count_leaf: key=[%s]\n",leaf->keyp);
printf("_bins_count_leaf: count_l=%d count_r=%d\n",leaf->count_l,leaf->count_r);
*/
	}
	return 0;
}

/*	35	*/
static char *_bins_get_data(datalen,datap)
int datalen;
char *datap;
{
	char *p1,*p2;
	int adlen;

	p2 = NULL;
	if (p1=datap) {
		if (akxt_get_gep_data(datalen,p1,NULL,&adlen) >= 0) {
/*
printf("_bins_get_data: adlen=%d\n",adlen);
*/
			p2 = Memdup(p1,adlen);
		}
	}
	return p2;
}

/*	36	*/
static tdtBINS_LEAF *_bins_make_leaf(keyp,aklen)
char *keyp;
int  aklen;
{
	tdtBINS_LEAF *leaf;

	if (leaf=(tdtBINS_LEAF *)Malloc(sizeof(tdtBINS_LEAF))) {
		memset(leaf,0,sizeof(tdtBINS_LEAF));
		if (!(leaf->keyp = Memdup(keyp,aklen))) {
			Free(leaf);
			leaf = NULL;
		}
	}
	return leaf;
}

/*	37	*/
static int _bins_r_key_klen(keylen,keyp1,klen,top_leaf,p_d,p_prev,p_leaf)
int keylen,klen,*p_d;
char *keyp1;
tdtBINS_LEAF *top_leaf,**p_prev,**p_leaf;
{
	int ret,d;
	char *p1,*p2;
	tdtBINS_LEAF *prev,*leaf,*next;

	if (!(p1=keyp1)) return -2;
	d = 0;
	prev = leaf = NULL;
	next = top_leaf;
	while (next) {
		prev = leaf;
		leaf = next;
/*
printf("_bins_r: leaf=%08x L=%08x R=%08x\n",
leaf,leaf->left,leaf->right);
printf("_bins_r: count_l=%d count_r=%d\n",leaf->count_l,leaf->count_r);
*/
		if (p2=leaf->keyp) {
/*
if (!keylen) printf("_bins_r: key=[%s]\n",p2);
*/
			d = akxt_cmp_gep_data(keylen,p1,klen,p2);
/*
printf("_bins_r:    d=%d\n",d);
*/
			if (d > 0) next = leaf->right;
			else if (d < 0) next = leaf->left;
			else {
				ret = 1;
				break;
			}
		}
		else return -4;
	}
	if (p_d)    *p_d    = d;
	if (p_prev) *p_prev = prev;
	if (p_leaf) *p_leaf = leaf;
	return ret;
}

/*	38	*/
static int _bins_r_key(keylen,keyp1,top_leaf,p_d,p_prev,p_leaf)
int keylen,*p_d;
char *keyp1;
tdtBINS_LEAF *top_leaf,**p_prev,**p_leaf;
{
	char *p1;
	int  klen;

	if (!(p1=keyp1)) return -2;
	if ((klen=akxt_get_gep_data(keylen,p1,&p1,NULL))<0) return -3;
/*
if (!keylen) printf("_bins_r_key: top_leaf=%08x key=[%s]\n",top_leaf,p1);
*/
	return _bins_r_key_klen(keylen,p1,klen,top_leaf,p_d,p_prev,p_leaf);
}

/*	39	*/
int _bins_proc(func,tph)
char  func;
HASHB *tph;
{
	int keylen,klen,d,aklen,adlen,ret;
	char *p1,*p2;
	tdtBINS_LEAF *prev,*leaf,*next,*next_r,*next_l;

	if (!tph) return -1;
	if (!(p1=tph->ha_key)) return -2;
	keylen = tph->ha_keylen;
	if ((klen=akxt_get_gep_data(keylen,p1,&p1,&aklen))<0) return -3;
/*
if (!keylen) printf("_bins_proc: func=%c key=[%s]\n",func,p1);
*/
	if (func=='S') {
		if (leaf=(tdtBINS_LEAF *)tph->ha_reg) {
			ret = _bins_s_key_klen(tph,p1,klen,aklen,&tph->ha_reg,leaf);
		}
		else {
			if (p1=(char *)tph->ha_next) {
				if (!(p2=_bins_get_data(tph->ha_maxreg,p1))) return -13;
			}
			else p2 = NULL;
			if (!(next=_bins_make_leaf(tph->ha_key,aklen))) return -12;
			next->datp = p2;
			leaf = next;
			tph->ha_reg = (char *)leaf;
/*
printf("_binsrs: s: %c leaf=%08x L=%08x R=%08x\n",
func,next,next->left,next->right);
*/
			ret = 0;
		}
		return ret;
	}

	next = (tdtBINS_LEAF *)tph->ha_reg;
	if ((ret=_bins_r_key_klen(keylen,p1,klen,next,&d,&prev,&leaf)) < 0)
		return ret;

	if (func=='D') {
		if (leaf && !d) {
			/* 폜ꂽleafNĂPOrightNB
			   rightNULL̂Ƃ́AleftNB*/
			next_r = leaf->right;
			next_l = leaf->left;
			if (next_r) next = next_r;
			else next = next_l;
/*
printf("_binsrs: d: %c prev=%08x leaf=%08x L=%08x R=%08x next=%08x\n",
func,prev,leaf,next_l,next_r,next);
*/
			if (prev) {
				/* POleafNĂnextN */
				if (prev->right == leaf)
					prev->right = next;
				else
					prev->left = next;
			}
			else {
				tph->ha_reg = (char *)next;
				prev = next;
			}
			/* rightNꂽ̂Aright͓o^B */
			if (next_r && next_l) {
				if (ret=_bins_s_leaf(keylen,next,next_l)) return ret;
			}
			_bins_count_leaf(prev);
			leaf->right = NULL;
			leaf->left  = NULL;
			_leaf_free(leaf);
			ret = 1;
		}
	}
	else {
		if (leaf && !d) {
			tph->ha_next = (int *)leaf->datp;
			ret = 1;
		}
	}
	return ret;
}

/*	40	*/
int _bins_s_leaf(keylen,top,aleaf)
int keylen;
tdtBINS_LEAF *top,*aleaf;
{
	int d,ret;
	char *p1;
	tdtBINS_LEAF *leaf;

	if (!(p1=aleaf->keyp)) return -2;
	if ((ret=_bins_r_key(keylen,p1,top,&d,NULL,&leaf))<0) return ret;
	if (leaf) {
		if (d > 0) leaf->right = aleaf;
		else if (d < 0) leaf->left = aleaf;
	}
/*	_bins_count_leaf(top);	*/
	return 0;
}

/*	41	*/
static int _bins_balance(keylen,pprev,leaf)
int keylen;
tdtBINS_LEAF **pprev,*leaf;
{
	int ret;
	tdtBINS_LEAF *next;

	if (X_DIFF(leaf->count_l,leaf->count_r) > 1) {
		if (leaf->count_l < leaf->count_r) {
			next = leaf->right;
			leaf->count_r = 0;
			leaf->right = NULL;
		}
		else {
			next = leaf->left;
			leaf->count_l = 0;
			leaf->left = NULL;
		}
		*pprev = next;
		ret = _bins_s_leaf(keylen,next,leaf);
	}
	return 0;
}

/*	42	*/
int _bins_s_key_klen(tph,keyp1,klen,aklen,pprev,leaf)
HASHB *tph;
int klen,aklen;
char *keyp1;
tdtBINS_LEAF **pprev,*leaf;
{
	int ret,d,count,keylen,datalen;
	char *p1,*p2;
	tdtBINS_LEAF **p_prev,*next;

	if (!tph || !keyp1) return -1;
	keylen = tph->ha_keylen;
	ret = 0;
	if (leaf) {
/*
printf("_bins_s_key_klen: leaf=%08x L=%08x R=%08x\n",
leaf,leaf->left,leaf->right);
*/
		if (p2=leaf->keyp) {
/*
if (!keylen) printf("_bins_s_key_klen: key=[%s]\n",p2);
*/
			d = akxt_cmp_gep_data(keylen,keyp1,klen,p2);
/*
printf("_bins_s_key_klen:    d=%d\n",d);
*/
			if (d) {
				if (d > 0) {
					next = leaf->right;
					p_prev = &leaf->right;
				}
				else if (d < 0) {
					next = leaf->left;
					p_prev = &leaf->left;
				}
				if (next) {
					ret= _bins_s_key_klen(tph,keyp1,klen,aklen,p_prev,next);
					count = next->count_a + 1;
					if (next == leaf->right)
						leaf->count_r = count;
					else
						leaf->count_l = count;
					leaf->count_a = leaf->count_l + leaf->count_r;
/*
if (!keylen) printf("_bins_s_key_klen: key=[%s]\n",p2);
printf("_bins_s_key_klen: count_l=%d count_r=%d\n",leaf->count_l,leaf->count_r);						ret = _bins_balance(keylen,pprev,leaf);
*/
					_bins_count_leaf(*pprev);
					return ret;
				}
			}
			if (p1=(char *)tph->ha_next) {
				if (!(p2=_bins_get_data(tph->ha_maxreg,p1))) return -13;
			}
			else p2 = NULL;
			if (d) {
				if (!(next=_bins_make_leaf(tph->ha_key,aklen))) return -12;
				next->datp = p2;
			}
			if (d > 0) {
				leaf->right = next;
				leaf->count_r = 1;
			}
			else if (d < 0) {
				leaf->left = next;
				leaf->count_l = 1;
			}
			else {
				if (leaf->datp) Free(leaf->datp);
				leaf->datp = p2;
				ret = 1;
			}
			leaf->count_a = leaf->count_l + leaf->count_r;
/*
printf("_bins_s_key_klen: leaf=%08x L=%08x R=%08x\n",
leaf,leaf->left,leaf->right);
*/
		}
		else return -4;
	}
	return ret;
}

/*	43	*/
int akxs_get_KMP_next(char *str, int str_len, int next[])
{
	int i,j,n;

	if (!str || !next) return -1;

	next[0] = -1;
	n = next[0];
	for (i=0;i<str_len;i++) {
		while (n >= 0) {
			if (str[i] == str[n]) break;
			 n = next[n];
		}
		n++;
		j = i + 1;
		if (str[j] == str[n]) next[j] = next[n];
		else next[j] = n;
	}
	return 0;
}

/*	44	*/
int akxs_find_KMP(char *text, int text_len, char *str, int str_len, int next[])
{
	int ret,text_idx_len,text_idx,n;

	if (!text || !str || !next) return -1;

	if (ret=akxs_get_KMP_next(str,str_len,next)) return ret;

	text_idx_len = text_len - str_len + 1;
	n = 0;
	for (text_idx=0;text_idx<text_idx_len;text_idx++) {
		while (n >= 0) {
			if (text[text_idx] == str[n]) break;
			n = next[n];
		}
		n++;
		if (str_len <= n) return text_idx - str_len + 2;
	}
	return 0;
}

/*	51	*/
/********************************************************************/
/*  @\ : Pz܂̓O[v\̔z񒆂̕܂͔ԍ	*/
/*		   T[`												*/
/*	*stra[]=														*/
/*		{"0","SYSCODE",""											*/
/*		,"1","EUC-JP","EUCJP","EUC",""								*/
/*		,"2","CP932","SJIS","S-JIS",""								*/
/*		,"3","ISO-2022-JP-3","JIS",""								*/
/*		,"4","CP939","EBCDIC",""									*/
/*		,"5","UTF-8","UTF8",""										*/
/*		,NULL};														*/
/*	 : IN : stra[]   : T[`Ώە|C^z				*/
/*						   1grouṕA󕶎ŏI					*/
/*				max_stra : strãT[`Ώۗvf					*/
/*							< 0 : NULLŏI						*/
/*				str      : T[`								*/
/*				opt      : IvV								*/
/*							= 0x10 : grouping						*/
/*							= 0x20 : O[v擪̓T[`Ȃ		*/
/*									 0x10ONƌȂ				*/
/*							= 0x01 : ignore case					*/
/*							= 0x02 : ignore Spp				*/
/*							= 0x04 : Sp					*/
/*							= 0x08 : Ov						*/
/*				posa[]  : vʒu									*/
/*							posa[1] : vsträʒu (擪1)	*/
/*						  opt=0x0X									*/
/*							posa[0] : 0								*/
/*						  opt=0x1X ̂								*/
/*							posa[0] : group擪sträʒu			*/
/*						  opt=0x2X									*/
/*							posa[0] : group no.(ŏgroup1)		*/
/*	ԋp : = 0 : vȂ										*/
/*		   > 0 : vsträʒu (擪1)						*/
/*				   opt=0x10̂ON̂ƂAgroup no. (ŏgroup1)	*/
/*				   opt=0x20ON̂ƂAgroup擪sträʒu		*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2019/08/23 Akito Kobayashi								*/
/*	XV : 2022/06/08 Akito Kobayashi								*/
/*	XV : 2022/11/01 Akito Kobayashi								*/
/********************************************************************/
int akxs_seqr_str_grp(stra,max_stra,str,opt,posa)
char *stra[],*str;
int max_stra,opt,posa[];
{
	int i,ii,ig,opt1,opt2,iNUM,i0,i3;
	int i2;	/* group擪sträʒu */
	char **pp,*p;
/*
printf("akxs_seqr_str: max_stra=%d opt=%08x\n",max_stra,opt);
*/
	if (!stra || !str) return -185215101;
	if (!max_stra) return 0;
	opt1 = opt & 0x10;
	opt2 = opt & 0x20;
	if (opt2) opt1 = 0x10;
	ii = 0;
	ig = 0;
	if (opt1) ig = 1;
	iNUM = 0;
	if (opt2) iNUM = 1;
/*
printf("akxs_seqr_str: opt1=%d opt2=%d iNUM=%d str=[%s]\n",opt1,opt2,iNUM,str);
*/
	if (posa) {
		posa[1] = posa[0] = 0;
	}
	i0 = 0;
	i2 = 1;
	for (i=0,pp=stra;;i++,pp++) {
		p = *pp;
		if (max_stra >= 0) {
			if (i >= max_stra) break;
		}
		else if (!p) break;
		if (p) {
/*
printf("akxs_seqr_str: i=%d ig=%d i0=%d p=[%s]\n",i,ig,i0,p);
*/
			if (opt1 && !*p) {
				ig++;
				i2 = i + 2;	/* i̎group擪łAi2stra̐擪1Ɛ̂2 */
				if (opt2) iNUM = 1;
			}
			else if (opt2 && iNUM) {
			/*	i0 = i + 1;	del 2022.11.01*/
				iNUM = 0;
			}
			else if (!akxs_opt_strcmp(p,str,opt)) {
/*
printf("akxs_seqr_str: found i=%d ig=%d\n",i,ig);
*/
#if 1	/* 2022.11.01 */
				if (opt1) {
					if (opt2) {
						ii = i2;
						i3 = ig;
					}
					else {
						ii = ig;
						i3 = i2;
					}
				}
				else {
					ii= i + 1;
					i3 = 0;
				}
				if (posa) {
					posa[0] = i3;
					posa[1] = i + 1;
				}
#else
				if (i0 > 0) ii = i0;
				else if (opt1) ii = ig;
				else ii= i + 1;
				if (posa) {
					posa[0] = ig;
					posa[1] = i + 1;
				}
#endif
				break;
			}
		}
	}
/*
printf("akxs_seqr_str: ii=%d\n",ii);
*/
	return ii;
}

/*	52	*/
int akxs_seqr_str(stra,max_stra,str,opt)
char *stra[],*str;
int max_stra,opt;
{
	return akxs_seqr_str_grp(stra,max_stra,str,opt,NULL);
}

/*	53	*/
/********************************************************************/
/*  @\ : O[v\̔z񒆂̔ԍT[`					*/
/*   : IN : stra[]   : T[`Ώە|C^z				*/
/*						   group\łA1group͋󕶎ŏI	*/
/*				max_stra : strãT[`Ώۗvf					*/
/*							< 0 : NULLŏI						*/
/*				str      : T[`								*/
/*				opt      : IvV								*/
/*							= 0x01 : ignore case					*/
/*							= 0x02 : ignore Spp				*/
/*							= 0x04 : Sp					*/
/*							= 0x08 : Ov						*/
/*  ԋp : = 0 : vȂ										*/
/*		   > 0 : vstra̔ԍ̈ʒu(擪1)				*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2019/08/23 Akito Kobayashi								*/
/********************************************************************/
int akxs_seqr_str_num(stra,max_stra,str,opt)
char *stra[],*str;
int max_stra,opt;
{
	int i,ii,iNUM;
	char **pp,*p;
/*
printf("akxs_seqr_str_num: max_stra=%d opt=%08x\n",max_stra,opt);
*/
	if (!stra || !str) return -185215301;
	if (!max_stra) return 0;
	iNUM = 1;
/*
printf("akxs_seqr_str_num: iNUM=%d str=[%s]\n",iNUM,str);
*/
	ii = 0;
	for (i=0,pp=stra;;i++,pp++) {
		p = *pp;
		if (max_stra >= 0) {
			if (i >= max_stra) break;
		}
		else if (!p) break;
		if (p) {
/*
printf("akxs_seqr_str_num: i=%d iNUM=%d p=[%s]\n",i,iNUM,p);
*/
			if (!*p) {
				iNUM = 1;
			}
			else if (iNUM) {
				if (!akxs_opt_strcmp(p,str,opt)) {
					ii = i + 1;
/*
printf("akxs_seqr_str_num: found i=%d ig=%d\n",i,ig);
*/
					break;
				}
				iNUM = 0;
			}
		}
	}
/*
printf("akxs_seqr_str_num: ii=%d\n",ii);
*/
	return ii;
}

/*	54	*/
/********************************************************************/
/*  @\ : ԍƕ̃yA̔z񒆂̕܂͔ԍ		*/
/*		   T[`												*/
/*		*stra[]={"1","AAAAA","3","BBBBB","4","CCCC",NULL}			*/
/*   : IN : stra[]   : T[`Ώە|C^z				*/
/*						   ԍƕ̃yA						*/
/*				max_stra : strãT[`Ώۗvf					*/
/*							< 0 : NULLŏI						*/
/*				str      : T[`܂͐					*/
/*				opt      : IvV								*/
/*							= 0x40 : ԍT[` 				*/
/*							= 0x80 : ԍ͐	 				*/
/*							= 0x01 : ignore case					*/
/*							= 0x02 : ignore Spp				*/
/*							= 0x04 : Sp					*/
/*							= 0x08 : Ov						*/
/*  ԋp : = 0 : vȂ										*/
/*		   > 0 : vstra̔ԍ̈ʒu(擪1)				*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2019/08/23 Akito Kobayashi								*/
/********************************************************************/
int akxs_seqr_str2(stra,max_stra,str,opt)
char *stra[],*str;
int max_stra,opt;
{
	int i,ii,ig,opt4,intval,i1,iNUM,i0,opt8,k;
	char **pp,*p,*p0,*p1;
/*
printf("akxs_seqr_str2: max_stra=%d opt=%08x\n",max_stra,opt);
*/
	if (!stra || !str) return -185215401;
	opt4 = opt & 0x40;
	opt8 = opt & 0x80;
	if (!max_stra) return 0;
	else if (max_stra > 0) {
		max_stra &= ~0x01;
		if (!max_stra) return 0;
	}
/*
printf("akxs_seqr_str2: str=[%s] max_stra=%d opt4=%d\n",str,max_stra,opt4);
*/
	ii = 0;
	for (i=0,pp=stra;;i+=2) {
		p  = *pp++;
		p1 = *pp++;
		if (max_stra >= 0) {
			if (i >= max_stra) break;
		}
		else if (!p) break;
		if (p) {
/*
printf("akxs_seqr_str2: i=%d p=[%s] p1=[%s]\n",i,p,p1);
*/
			if (opt4) p1 = p;
/*
printf("akxs_seqr_str: p1=[%s]\n",p1);
*/
			if (opt8) {
				k = !(str == p1);
			}
			else {
				k = akxs_opt_strcmp(p1,str,opt);
			}
			if (!k) {
/*
printf("akxs_seqr_str2: found i=%d ig=%d\n",i,ig);
*/
				ii= i + 1;
				break;
			}
		}
	}
/*
printf("akxs_seqr_str2: ii=%d\n",ii);
*/
	return ii;
}

/*	56	*/
/********************************************************************/
/*  @\ : Integerz񒆂T[`								*/
/*   : IN : ia[]   : T[`ΏIntegerz						*/
/*				max_ia : iãT[`Ώۗvf						*/
/*				val    : T[`Integerl							*/
/*				opt    : rIvV								*/
/*						 = 0 : ==									*/
/*						 = 1 : &									*/
/*						 = 2 : !=									*/
/*						 = 3 : Ȃől						*/
/*							   ԋpl= 0 : 擪val𒴂Ă	*/
/*  ԋp : = 0 : vȂ										*/
/*		   > 0 : viäʒu(擪1)							*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2019/09/09 Akito Kobayashi								*/
/********************************************************************/
int akxs_iseq_opt(ia,max_ia,val,opt)
int ia[],max_ia,val,opt;
{
	int i,ii,iai,match;

	if (!ia) return -185215601;
	match = ii = 0;
	for (i=0;i<max_ia;i++) {
		iai = ia[i];
		if (!opt) {
			if (iai == val) match = 1;
		}
		else if (opt == 1) {
			if (iai & val) match = 1;
		}
		else if (opt == 2) {
			if (iai != val) match = 1;
		}
		else if (opt == 3) {
			if (iai > val) {
				ii = i;
				break;
			}
		}
		if (match) {
			ii = i + 1;
			break;
		}
	}
	if (opt==3 && i>=max_ia) ii = max_ia;
	return ii;
}

/*	57	*/
int akxs_iseq(ia,max_ia,val)
int ia[],max_ia,val;
{
	return akxs_iseq_opt(ia,max_ia,val,0);
}

/*	58	*/
/********************************************************************/
/*  @\ : ̎wvfʒuɕvf}				*/
/*   : IN : da     : }敶ւ̃|C^					*/
/*				d_pos  : }vfʒu (擪0)						*/
/*						   <0̂Ƃ́A0ƌȂ					*/
/*				max_da : }敶̗vf						*/
/*						   <0̂Ƃ́A0ƌȂ					*/
/*				s_val  : }vfւ̃|C^				*/
/*				size   : vfoCg								*/
/*  ԋp : > 0 : }̗vf (max_da+1)							*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2019/09/09 Akito Kobayashi								*/
/********************************************************************/
int akxs_ins_sseq(da,d_pos,max_da,s_val,size)
char *da,*s_val;
int   d_pos,max_da,size;
{
	int i;
	char *pd;

	if (!da || !s_val) return -185215801;
	if (max_da < 0) max_da = 0;
	if (d_pos < 0) d_pos = 0;
	pd = da + (max_da - 1)*size;
	if (max_da>0 && d_pos>=0 && d_pos<max_da) {
		for (i=d_pos;i<max_da;i++,pd-=size) {
			memcpy(pd+size,pd,size);
		}
	}
	memcpy(pd+size,s_val,size);
	return ++max_da;
}

/*	61	*/
/********1*********2*********3*********4*********5*********6*********/
/*  @\ : ̎wvfʒuɕvf}				*/
/*   : IN : da     : }敶ւ̃|C^					*/
/*				d_pos  : }vfʒu (擪0)					*/
/*						   <0̂Ƃ́A0ƌȂ					*/
/*				max_da : }敶̗vf						*/
/*						   <0̂Ƃ́A0ƌȂ					*/
/*				sa     : }vfւ̃|C^				*/
/*				s_pos  : }vfʒu (擪0)					*/
/*				n_sa   : }镶̗vf						*/
/*						   <=0̂Ƃ́AȂ					*/
/*				size   : vfoCg								*/
/*				opt    : }vfNAIvV					*/
/*						   <=0 : NAȂ						*/
/*						   = 1 : pXy[XŃNA			*/
/*						   > 1 : [ŃNA					*/
/*  ԋp : > 0 : }̗vf (n_sa)						*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2019/09/09 Akito Kobayashi								*/
/********************************************************************/
int akxs_ins_sseq_array(da,d_pos,max_da,sa,s_pos,n_sa,size,opt)
char *da,*sa;
int   d_pos,max_da,s_pos,n_sa,size,opt;
{
	int i;
	char *ps,pad;

	if (!da || !sa) return -185216101;
	if (s_pos < 0) s_pos = 0;
	if (opt == 1) pad = ' ';
	else if (opt > 1)pad = '\0';
	ps = sa + s_pos*size;
	for (i=0;i<n_sa;i++) {
		max_da = akxs_ins_sseq(da,d_pos,max_da,ps,size);
		if (opt > 0) memset(ps,pad,size);
		d_pos++;
		ps += size;
	}
	return n_sa;
}

/*	62	*/
/********************************************************************/
/*  @\ : shortz̎wvfʒushortvf}				*/
/*   : IN : da     : }shortzւ̃|C^				*/
/*				d_pos  : }vfʒu (擪0)					*/
/*						   <0̂Ƃ́A0ƌȂ					*/
/*				max_da : }shortz̗vf					*/
/*						   <0̂Ƃ́A0ƌȂ					*/
/*				s_val  : }shortl								*/
/*  ԋp : > 0 : }̗vf (max_da+1)							*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2019/09/09 Akito Kobayashi								*/
/********************************************************************/
int akxs_ins_shseq(da,d_pos,max_da,s_val)
short da[],s_val;
int   d_pos,max_da;
{
#if 1
	return akxs_ins_sseq(da,d_pos,max_da,&s_val,sizeof(short));
#else
	int i;
	short *pd;

	if (max_da < 0) max_da = 0;
	if (d_pos < 0) d_pos = 0;
	pd = da + max_da - 1;
	if (max_da>0 && d_pos>=0 && d_pos<max_da) {
		for (i=d_pos;i<max_da;i++,pd--) {
			*(pd+1) = *pd;
		}
	}
	*(pd+1) = s_val;
	return ++max_da;
#endif
}

/*	63	*/
/********1*********2*********3*********4*********5*********6*********/
/*  @\ : shortz̎wvfʒushortzvf}			*/
/*   : IN : da     : }shortzւ̃|C^				*/
/*				d_pos  : }vfʒu (擪0)					*/
/*						   <0̂Ƃ́A0ƌȂ					*/
/*				max_da : }shortz̗vf					*/
/*						   <0̂Ƃ́A0ƌȂ					*/
/*				sa     : }shortzvfւ̃|C^			*/
/*				s_pos  : }vfʒu (擪0)					*/
/*				n_sa   : }镶̗vf						*/
/*						   <=0̂Ƃ́AȂ					*/
/*				size   : vfoCg								*/
/*				opt    : }vfNAIvV					*/
/*						   <=0 : NAȂ						*/
/*						   >=1 : [ŃNA					*/
/*  ԋp : > 0 : }̗vf (n_sa)						*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2019/09/09 Akito Kobayashi								*/
/********************************************************************/
int akxs_ins_shseq_array(da,d_pos,max_da,sa,s_pos,n_sa,opt)
short da[],sa[];
int   d_pos,max_da,s_pos,n_sa,opt;
{
#if 1
	if (opt == 1) opt = 2;
	return akxs_ins_sseq_array(da,d_pos,max_da,sa,s_pos,n_sa,sizeof(short),opt);
#else
	int i;
	short *ps;

	if (s_pos < 0) s_pos = 0;
	ps = sa + s_pos;
	for (i=0;i<n_sa;i++) {
		max_da = akxs_ins_shseq(da,d_pos,max_da,*ps);
		if (opt) *ps = 0;
		d_pos++;
		ps++;
	}
	return n_sa;
#endif
}

/*	66	*/
/********1*********2*********3*********4*********5*********6*********/
/*  @\ : [̕sPʂŃT[`					*/
/*   : IN : pat     : T[`镶ւ̃|C^				*/
/*				pat_len : T[`镶̒(oCg)			*/
/*				mem     : T[`Ώەւ̃|C^				*/
/*				mem_len : T[`Ώە̒(oCg)			*/
/*  ԋp : > 0 : patɈvs̐擪ʒu(mem̐擪1oCgڂƂ)*/
/*		   < 0 : |C^NULL										*/
/*	쐬 : 2020/12/XX Akito Kobayashi								*/
/********************************************************************/
int akxs_sseq_mline(pat,patlen,mem,memlen)
char *pat,*mem;
int patlen,memlen;
{
	SSP_S ssp;
	int ret,sp,len;

	if (!pat || !mem) return -185216601;
	ret = 0;
/*
printf("akxs_sseq_mline: patlen=%d pat=[%s]\n",patlen,pat);
*/
	if (patlen>0 && memlen>=patlen) {
		memset(&ssp,0,sizeof(SSP_S));
		sp = 0;
		while ((len=akxtmgetline(mem, memlen, &ssp)) >= 0 ) {
/*
printf("akxs_sseq_mline: len=%d ssp.wd=[%s]\n",len,strmem(ssp.wd,len));
*/
			if (len==patlen && !memcmp(ssp.wd,pat,patlen)) {
				ret = sp + 1;
				break;
			}
			sp = ssp.sp;
		}
	}
	return ret;
}

/*	71	*/
XHASHB *akxs_xseqlm_new(max_reg,m_alloc,pConstCt)
int max_reg;
char *(*m_alloc)();
tdtCONSTCT *pConstCt;
{
	XHASHB *p;

	if (!(p=(XHASHB *)akxm_malloc_constct(m_alloc,pConstCt,sizeof(HASHB)))) {
		errno = -1303;
		return NULL;
	}
	return 0;
}

/*	72	*/
HASHB *akxs_seqlm_new(max_reg,m_alloc,pConstCt)
int max_reg;
char *(*m_alloc)();
tdtCONSTCT *pConstCt;
{
	HASHB *hp;
	char *p;
	int len;

	len = sizeof(HASHB) + max_reg*(sizeof(long)+1) + 1;
	if (!(p=akxm_malloc_constct(m_alloc,pConstCt,len))) {
		errno = -185215201;
		return NULL;
	}
	memset(p,0,len);
	hp = (HASHB *)p;
	hp->ha_id[0] = 'S';
	hp->ha_id[1] = 'L';
	hp->ha_maxreg = max_reg;
	p += sizeof(HASHB);
	hp->ha_reg = p;
	p += max_reg*sizeof(long);
	hp->ha_key = p;
	akxs_seql_i(hp);
	return hp;
}

/*	73	*/
int akxs_seql_i(ha)
HASHB *ha;
{
	int max_reg;

	if ((max_reg=ha->ha_maxreg) <= 0) return -185215304;
	memset(ha->ha_reg,0,max_reg *sizeof(long));
	memset(ha->ha_key,0,max_reg);
	return 0;
}

/*	74	*/
int akxs_seql_r(ha,lVal)
HASHB *ha;
long lVal;
{
	int i,ii;
	long *pV;
	char *p;

	ii = 0;
	pV = (long *)ha->ha_reg;
	p  = ha->ha_key;
	for (i=0;i<ha->ha_maxreg;i++,p++,pV++) {
		if (*p && *pV==lVal) {
			ii = i + 1;
			break;
		}
	}
	return ii;
}

/*	75	*/
int akxs_seql_s(ha,lVal)
HASHB *ha;
long lVal;
{
	int i,ii;
	long *pV;
	char *p;

	if (!(ii = akxs_seql_r(ha,lVal))) {
		p = ha->ha_key;
		i = akxstrnlen(p,ha->ha_maxreg);
		if (i < ha->ha_maxreg) {
			pV = (long *)ha->ha_reg;
			pV[i] = lVal;
			*(p+i) = '1';
			ii = i + 1;
		}
	}
	return ii;
}

/*	76	*/
int akxs_seql_d(ha,lVal)
HASHB *ha;
long lVal;
{
	int i,ii;
	long *pV;
	char *p;

	if ((ii=akxs_seql_r(ha,lVal)) > 0) {
		i = ii - 1;
		pV = (long *)ha->ha_reg;
		pV[i] = 0;
		p = ha->ha_key;
		*(p+i) = '\0';
	}
	return ii;
}
